/**
 * File for load simulation logic.
 */


// User Import Headers.
#ifndef user_headers
    #define user_headers
    #include "simulate_loads.h"
    #include "structs.h"
    #include "terminal_commands.h"
#endif


/**
 * Logic to run "Asynchronous Round Robin" load scheme.
 */
void run_arr(thread_struct* thread_args_ptr) {
    int process_rank = thread_args_ptr->thread_num;

    if (process_rank == 0) {
        printf("Starting ARR load scheme.\n");
        printf("\n");
    }

    // Wait for all processes to synchronize.
    // Also sleep for a second to allow time for console output to process.
    MPI_Barrier(MPI_COMM_WORLD);
    sleep(1);

    // Handle based on process number.
    if (process_rank > 0) {
        // Child process. Handles all the work.

        int* send_array = calloc(2, sizeof(int));
        send_array[0] = process_rank;
        send_array[1] = thread_args_ptr->total_loads;
        MPI_Send(send_array, 2, MPI_INT, 0, 0, MPI_COMM_WORLD);
        free(send_array);

    } else {
        // Main process. Handles communication to terminal.

        // Get message from all child processors.
        int index = 0;
        while (index < thread_args_ptr->total_processors - 1) {

            int* recv_array = calloc(3, sizeof(int));
            MPI_Recv(recv_array, 2, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            printf("Thread %i: %i / %i Loads Remaining\n", recv_array[0], recv_array[1], thread_args_ptr->total_loads);
            free(recv_array);

            index += 1;
        }
    }

    // Wait for all processes to synchronize.
    // Also sleep for a second to allow time for console output to process.
    MPI_Barrier(MPI_COMM_WORLD);
    sleep(1);

    if (process_rank == 0) {
        printf("\n");
        printf("ARR load scheme. Finished.\n");
    }
}


/**
 * Logic to run "Global Round Robin" load scheme.
 */
void run_grr(int total_processors) {
    printf("Starting GRR load scheme.");
    printf("\n");

    // pthread_t* thread_pool = NULL;

    printf("\n");
    printf("GRR load scheme. Finished.");
}


/**
 * Logic to run "Random Poling" load scheme.
 */
void run_rp(int total_processors) {
    printf("Starting RP load scheme.");
    printf("\n");

    // pthread_t* thread_pool = NULL;

    printf("\n");
    printf("RP load scheme. Finished.");
}



/**
 * Logic to run "Nearest Neighbor" load scheme.
 */
void run_nn(int total_processors) {
    printf("Starting NN load scheme.");
    printf("\n");

    // pthread_t* thread_pool = NULL;

    printf("\n");
    printf("NN load scheme. Finished.");
}


// /**
//  * Function to simulate working on a load.
//  */
// void simulate_load(char* thread_name) {
//     printf("Simulating load processing on thread \"%s\".\n", thread_name);

//     int load_counter = 0;
//     int index = 0;

//     // Iterate through loads. Each load has indexes to simulate "work".
//     while (load_counter < total_loads) {

//         // Increment load num.
//         load_counter += 1;
//         printf("Simulating load %i on thread \"%s\".\n", load_counter, thread_name);

//         // Simulate running the load.
//         index = 0;
//         while (index < indexes_per_load) {

//             // Print out index value.
//             if ((index % 10) == 0) {
//                 printf("load %i / %i    index %i / %i\n", load_counter, total_loads, index, indexes_per_load);
//             }

//             // Wait 1 second to simulate work.
//             sleep(1);

//             // Increment index.
//             index += 1;
//         }

//     }

// }


/**
 * Function to simulate working on an ARR load.
 */
// void* simulate_arr_load(thread_struct* thread_struct_ptr) {
//     printf("Simulating load processing on thread \"%i\".\n", thread_struct_ptr->thread_num);

//     // Send message back to main thread.
//     // char* message = "Thread %i checking in!\n";
//     // MPI_Send(message, sizeof(message) + 1, MPI_CHAR, 0, 0,)

//     // int load_counter = 0;
//     // int index = 0;

//     // // Iterate through loads. Each load has indexes to simulate "work".
//     // while (load_counter < total_loads) {

//     //     // Increment load num.
//     //     load_counter += 1;
//     //     printf("Simulating load %i on thread \"%s\".\n", load_counter, thread_name);

//     //     // Simulate running the load.
//     //     index = 0;
//     //     while (index < indexes_per_load) {

//     //         // Print out index value.
//     //         if ((index % 10) == 0) {
//     //             printf("load %i / %i    index %i / %i\n", load_counter, total_loads, index, indexes_per_load);
//     //         }

//     //         // Wait 1 second to simulate work.
//     //         sleep(1);

//     //         // Increment index.
//     //         index += 1;
//     //     }

//     // }

//     free(thread_struct_ptr);
// }

